// OBJECT Class
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// based on code by  by Alan Baylis 2001
// under GPL or new BSD license

#include <kos.h>

#include "object.h"


OBJECT::OBJECT()
{
Reset();
}

OBJECT::~OBJECT()
{
}

GLvoid OBJECT::Reset()
{
    Orientation.Reset();
    Position.Reset();
    Matrix.LoadIdentity();
    Delta_x = 0.0f;
    Delta_y = 0.0f;
    Delta_z = 0.0f;
    Movement_x = 0.0f;
    Movement_y = 0.0f;
    Movement_z = 0.0f;
 	Multiplier=1.0f;
}



GLvoid OBJECT::Rotate()
{

   Orientation.EulerToQuat(-Delta_x * radian * Multiplier, -Delta_y * radian* Multiplier , -Delta_z * radian * Multiplier);


}

GLvoid OBJECT::Rotate(GLfloat x, GLfloat y, GLfloat z)
{

   Orientation.EulerToQuat(x * radian* Multiplier , y * radian * Multiplier, z * radian* Multiplier);

}

GLvoid  OBJECT::Draw()
{
    // Should probably be a pure virtual
}


GLvoid OBJECT::Update()
{

    //neue position
    UpdatePosition();
    //rotiere
    Rotate();


}


GLvoid OBJECT::Apply()
{
    // erst translate dann rotate!

    glTranslatef(Position.x,Position.y,Position.z);
    //ab hier die modele ausrichten beim start nch vorne schauen
    glRotatef(-90, 0.0f, 1.0f, 0.0f);

    //rotiere nun die QUAD sachen
    Orientation.CreateMatrix(Matrix._m);
    glMultMatrixf(Matrix._m);

}


GLvoid OBJECT::SetPosition(GLfloat x, GLfloat y, GLfloat z)
{
    Position.x=x;
    Position.y=y;
    Position.z=z;
}

GLvoid  OBJECT::GetPosition(GLfloat *x, GLfloat *y, GLfloat *z){
    assert(x!=NULL);
    assert(y!=NULL);
    assert(z!=NULL);

    *x=Position.x;
    *y=Position.y;
    *z=Position.z;

}


GLvoid OBJECT::UpdatePosition()
{
    if (Movement_x != 0)
        MoveX();
    if (Movement_y != 0)
        MoveY();
    if (Movement_z != 0)
        MoveZ();

    Movement_x = 0.0f;
    Movement_y = 0.0f;
    Movement_z = 0.0f;
}

GLvoid OBJECT::UpdatePosition(GLfloat x, GLfloat y, GLfloat z)
{
    if (x != 0)
        MoveX(x);
    if (y != 0)
        MoveY(y);
    if (z != 0)
        MoveZ(z);
}



GLvoid OBJECT::MoveX()
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W;
    GLfloat X;
    GLfloat Y;
    GLfloat Z;
    QUAT TempQuat;
    QUAT TempQuat2;
    TempQuat = Orientation;
    TempQuat2.EulerToQuat(0.0f, -90.0f*radian, 0.0f);
    TempQuat.MultQuat(TempQuat2);
    W = TempQuat.w;
    X = TempQuat.x;
    Y = TempQuat.y;
    Z = TempQuat.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f - 2.0f * ( X * X + Y * Y );
    Position.x += DirX * Movement_x * Multiplier;
    Position.y += DirY * Movement_x * Multiplier;
    Position.z += DirZ * Movement_x * Multiplier;
}

GLvoid OBJECT::MoveY()
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W;
    GLfloat X;
    GLfloat Y;
    GLfloat Z;
    QUAT TempQuat;
    QUAT TempQuat2;
    TempQuat = Orientation;
    TempQuat2.EulerToQuat(90.0f*radian, 0.0f, 0.0f);
    TempQuat.MultQuat(TempQuat2);
    W = TempQuat.w;
    X = TempQuat.x;
    Y = TempQuat.y;
    Z = TempQuat.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f - 2.0f * ( X * X + Y * Y );
    Position.x += DirX * Movement_y * Multiplier;
    Position.y += DirY * Movement_y * Multiplier;
    Position.z += DirZ * Movement_y * Multiplier;
}

GLvoid OBJECT::MoveZ()
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W = Orientation.w;
    GLfloat X = Orientation.x;
    GLfloat Y = Orientation.y;
    GLfloat Z = Orientation.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f - 2.0f * ( X * X + Y * Y );
    Position.x += DirX * Movement_z * Multiplier;
    Position.y += DirY * Movement_z * Multiplier;
    Position.z += DirZ * Movement_z * Multiplier;
}

GLvoid OBJECT::MoveX(GLfloat x)
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W;
    GLfloat X;
    GLfloat Y;
    GLfloat Z;
    QUAT TempQuat;
    QUAT TempQuat2;
    TempQuat = Orientation;
    TempQuat2.EulerToQuat(0.0f, -90.0f*radian, 0.0f);
    TempQuat.MultQuat(TempQuat2);
    W = TempQuat.w;
    X = TempQuat.x;
    Y = TempQuat.y;
    Z = TempQuat.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f - 2.0f * ( X * X + Y * Y );
    Position.x += DirX * x* Multiplier;
    Position.y += DirY * x* Multiplier;
    Position.z += DirZ * x* Multiplier;
}

GLvoid OBJECT::MoveY(GLfloat y)
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W;
    GLfloat X;
    GLfloat Y;
    GLfloat Z;
    QUAT TempQuat;
    QUAT TempQuat2;
    TempQuat = Orientation;
    TempQuat2.EulerToQuat(90.0*radian, 0.0, 0.0);
    TempQuat.MultQuat(TempQuat2);
    W = TempQuat.w;
    X = TempQuat.x;
    Y = TempQuat.y;
    Z = TempQuat.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f - 2.0f * ( X * X + Y * Y );
    Position.x += DirX * y* Multiplier;
    Position.y += DirY * y* Multiplier;
    Position.z += DirZ * y* Multiplier;
}

GLvoid OBJECT::MoveZ(GLfloat z)
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W = Orientation.w;
    GLfloat X = Orientation.x;
    GLfloat Y = Orientation.y;
    GLfloat Z = Orientation.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f- 2.0f * ( X * X + Y * Y );
    Position.x += DirX * z* Multiplier;
    Position.y += DirY * z* Multiplier;
    Position.z += DirZ * z* Multiplier;
}

VECTOR OBJECT::GetXUnit()
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W;
    GLfloat X;
    GLfloat Y;
    GLfloat Z;
    QUAT TempQuat;
    QUAT TempQuat2;
    TempQuat = Orientation;
    TempQuat2.EulerToQuat(0.0f, -90.0f*radian, 0.0f);
    TempQuat.MultQuat(TempQuat2);
    W = TempQuat.w;
    X = TempQuat.x;
    Y = TempQuat.y;
    Z = TempQuat.z;
    DirX = 2.0f * ( X * Z - W * Y );
    DirY = 2.0f * ( Y * Z + W * X );
    DirZ = 1.0f - 2.0f * ( X * X + Y * Y );
    VECTOR Unit;
    Unit.x += DirX * 1;
    Unit.y += DirY * 1;
    Unit.z += DirZ * 1;
    return Unit;
}

VECTOR OBJECT::GetYUnit()
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W;
    GLfloat X;
    GLfloat Y;
    GLfloat Z;
    QUAT TempQuat;
    QUAT TempQuat2;
    TempQuat = Orientation;
    TempQuat2.EulerToQuat(90.0*radian, 0.0, 0.0);
    TempQuat.MultQuat(TempQuat2);
    W = TempQuat.w;
    X = TempQuat.x;
    Y = TempQuat.y;
    Z = TempQuat.z;
    DirX = 2.0 * ( X * Z - W * Y );
    DirY = 2.0 * ( Y * Z + W * X );
    DirZ = 1.0 - 2.0 * ( X * X + Y * Y );
    VECTOR Unit;
    Unit.x += DirX * 1;
    Unit.y += DirY * 1;
    Unit.z += DirZ * 1;
    return Unit;
}

VECTOR OBJECT::GetZUnit()
{
    GLfloat DirX;
    GLfloat DirY;
    GLfloat DirZ;
    GLfloat W = Orientation.w;
    GLfloat X = Orientation.x;
    GLfloat Y = Orientation.y;
    GLfloat Z = Orientation.z;
    DirX = 2.0 * ( X * Z - W * Y );
    DirY = 2.0 * ( Y * Z + W * X );
    DirZ = 1.0 - 2.0 * ( X * X + Y * Y );
    VECTOR Unit;
    Unit.x += DirX * 1;
    Unit.y += DirY * 1;
    Unit.z += DirZ * 1;
    return Unit;
}

