
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license

#include <kos.h>
#include <dc/ta.h>
#include <dcutils/pvrutils.h>
#include <jpeg/jpeg.h>
#include <pcx/pcx.h>

/* load a binary file into a texture */
void bin_to_texture(char *filename, uint32 tex, int w, int h)
{
    unsigned int fd;
    unsigned short *temp_tex;
    int size;

    temp_tex = ta_txr_map(tex);
    fd = fs_open(filename, O_RDONLY);
    size = fs_total(fd);
    if (size > (w*h*2))
        size = w*h*2;
    fs_read(fd, temp_tex, size);
    fs_close(fd);
}

/* Load a texture using pcx_load_texture and twiddle it */
void pcx_to_txr(const char *fn, uint32 *tex_out,int *tsize) {
    kos_img_t img;
    pvr_ptr_t txaddr;

    if (pcx_to_img(fn, &img) < 0) {
        printf("can't load %s\n", fn);
        return;
    }


    int size;

    txaddr = pvr_mem_malloc(img.w * img.h * 2);
    pvr_txr_load_kimg(&img, txaddr, PVR_TXRLOAD_INVERT_Y);


    if (tex_out != NULL) *tex_out = (uint32)txaddr;

    if(img.w>img.h) size=img.w;
    else size=img.h;

    if(tsize!=NULL) *tsize=img.w;

    kos_img_free(&img, 0);

}


void jpeg_to_txr(const char * filename, uint32* tex_out,
                 int alpha,int twiddle, int scale, int *tsize)
{

    kos_img_t img;
    pvr_ptr_t	tex;

    if (jpeg_to_img(filename,scale,&img)<0) {
        assert_msg(0,"Can't load jpeg texture\r\n");
        return;
    }

    printf("loading %s with w=%d and h=%d\n", filename,img.w,img.h);



    int size;

    if(img.w>img.h) size=img.w;
    else size=img.h;

    if(alpha) txr_to_alpha(img.data, size*scale,size*scale);

    if(!twiddle)
        tex = pvr_mem_malloc(size*size);
    else
        tex = pvr_mem_malloc(size*size*2);


    if(!twiddle)
        pvr_txr_load_ex(tex, img.data, size, size, PVR_TXRLOAD_16BPP);
    else
        /* Twiddle it into PVR mem as SIZExSIZE */
        txr_twiddle_scale(img.data, img.w, img.h,
                          tex, size, size);

    if (tex_out != NULL) *tex_out = (uint32)tex;
    if(tsize!=NULL) *tsize=size;

    /* Free the image */
    kos_img_free(&img, 0);

}

/* optimal random number function. */
static unsigned long seed=123;
#define RNDC 1013904223
#define RNDM 1164525

inline static int rand() {
    seed = seed * RNDM + RNDC;
    return seed;
}


int randnum(int limit) { return (rand() & 0x7fffffff) % limit; }


void my_memclear(char *ptr,int size){

    int i;

    for(i=0;i<size;i++,ptr++) *ptr=0;

}
